@@ -9,7 +9,8 @@ class Mean(keras.metrics.Metric):
             self,
             function: Callable[[Any, Any, Any], tf.Tensor],
             name: str,
-            dtype: tf.dtypes.DType = tf.float32) -> None:
+            dtype: tf.dtypes.DType = tf.float32,
+    ) -> None:
 
         super().__init__(name=name, dtype=dtype)
 
@@ -26,7 +27,7 @@ class Mean(keras.metrics.Metric):
 
     def update_state(self, y_true, y_pred, sample_weight: Optional[tf.Tensor] = None) -> None:
         values = self.function(y_true, y_pred, sample_weight)
-        batch_size = tf.cast(tf.shape(values)[0], self.dtype)
+        batch_size = tf.cast(tf.size(values), self.dtype)
 
         self.update_mean(values, batch_size)
         self.count.assign_add(batch_size)
