@@ -1,7 +1,7 @@
 import json
 import os
 from abc import ABC
-from typing import List, Dict, Mapping, Tuple, Optional, TypeVar, Callable, Iterable, Any
+from typing import List, Dict, Mapping, Tuple, Optional, TypeVar, Union, Callable, Iterable, Any
 
 import sfu_data_io.helpers as io
 import tensorflow as tf
@@ -9,16 +9,17 @@ from tensorflow import keras
 from tensorflow.keras.callbacks import Callback
 
 import sfu_ml_lib.base.metrics as metric_helpers
-from sfu_ml_lib.base.dataset import BatchType
+from sfu_ml_lib.base.dataset import DatasetType
 from sfu_ml_lib.base.metrics import Metric, MetricKeyType, NumericType
 from sfu_ml_lib.metrics.aggregator import KerasMetric
-from sfu_ml_lib.models.schema import Schema
 
 
 T = TypeVar('T')
 W = TypeVar('W')
 Z = TypeVar('Z')
-BatchFunction = Callable[[], Iterable[BatchType]]
+BatchFunction = Callable[[], DatasetType]
+Specification = Union[tf.SparseTensorSpec, tf.RaggedTensorSpec, tf.TensorSpec]
+Schema = Union[Specification, Iterable['Schema']]  # type: ignore
 
 
 class KerasModel(keras.Model, ABC):
@@ -79,14 +80,13 @@ class KerasModel(keras.Model, ABC):
         for metric in self.metrics_copy:
             metric.reset_states()
 
-    def create_tf_dataset(self, batches: BatchFunction, queue_size: int = -1) -> tf.data.Dataset:
-        tensorflow_dataset = tf.data.Dataset.from_generator(
-            generator=batches,
-            output_types=self.input_schema.get_types(),
-            output_shapes=self.input_schema.get_shapes(),
-        )
+    def create_tf_dataset(self, batch_generator: BatchFunction, queue_size: int = -1) -> tf.data.Dataset:
+        batches = batch_generator()
 
-        return tensorflow_dataset.prefetch(queue_size)
+        if not isinstance(batches, tf.data.Dataset):
+            batches = tf.data.Dataset.from_generator(batch_generator, output_signature=self.input_schema)
+
+        return batches.prefetch(queue_size)
 
     @tf.function
     def _train_dataset(self, dataset: tf.data.Dataset) -> Mapping[str, tf.Tensor]:
@@ -154,7 +154,8 @@ class KerasModel(keras.Model, ABC):
             max_num_epochs: int = 1,
             validation_batches: Optional[BatchFunction] = None,
             test_batches: Optional[BatchFunction] = None,
-            callbacks: Optional[List[Callback]] = None) -> None:
+            callbacks: Optional[List[Callback]] = None,
+    ) -> None:
         """
         Please look at `train_dataset` for details on how to construct `batches`.
         """
@@ -197,7 +198,8 @@ class KerasModel(keras.Model, ABC):
             loss_weights=None,
             sample_weight_mode=None,
             weighted_metrics=None,
-            **kwargs) -> None:
+            **kwargs,
+    ) -> None:
         """
         There's a bug in TensorFlow 2.1 that does not set the `metrics` property when compiling.
         So we create our own. This is a hack that should be removed as soon as they fix it.
